"""The clearmenu commands module.

These commands are used in the following models:
DPO2K, DPO2KB, DPO4K, DPO4KB, MDO3, MDO3K, MDO4K, MDO4KB, MDO4KC, MSO2K, MSO2KB, MSO4K, MSO4KB

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:

::

    - CLEARMenu
"""
from typing import Optional, TYPE_CHECKING

from .._helpers import SCPICmdWriteNoArguments

if TYPE_CHECKING:
    from tm_devices.drivers.pi.pi_device import PIDevice


class Clearmenu(SCPICmdWriteNoArguments):
    """The ``CLEARMenu`` command.

    **Description:**
        - Clears the current menu from the display. This command is equivalent to pressing the front
          panel Menu off.

    **Usage:**
        - Using the ``.write()`` method will send the ``CLEARMenu`` command.

    **SCPI Syntax:**

    ::

        - CLEARMenu
    """

    def __init__(self, device: Optional["PIDevice"] = None, cmd_syntax: str = "CLEARMenu") -> None:
        super().__init__(device, cmd_syntax)



"""Base REST Application Programming Interface (API) device driver module."""
import json
import time

from abc import ABC, abstractmethod
from types import MappingProxyType
from typing import Any, cast, Dict, Mapping, Optional, Tuple, Union

import requests

from tm_devices.drivers.api.api_device import APIDevice
from tm_devices.helpers import DeviceConfigEntry, print_with_timestamp, SupportedRequestTypes


class RESTAPIDevice(APIDevice, ABC):
    """Base REST Application Programming Interface (API) device driver."""

    API_VERSIONS: "MappingProxyType[int, str]" = MappingProxyType({})
    """Dict[int, str]: A mapping of supported API version numbers with the corresponding API URL."""

    ################################################################################################
    # Magic Methods
    ################################################################################################
    def __init__(self, config_entry: DeviceConfigEntry, verbose: bool) -> None:
        """Create a REST API device.

        Args:
            config_entry: A config entry object parsed by the DMConfigParser.
            verbose: A boolean indicating if verbose output should be printed.
        """
        super().__init__(config_entry, verbose)

        # The URL to use for REST API requests
        self._base_url = f"https://{self.ip_address}"
        self._api_url = ""

    ################################################################################################
    # Properties
    ################################################################################################
    @property
    def api_url(self) -> str:
        """Return the base API url of the device."""
        return self._api_url

    @property
    def base_url(self) -> str:
        """Return the base url of the device."""
        return self._base_url

    ################################################################################################
    # Public Methods
    ################################################################################################
    def delete(  # noqa: PLR0913
        self,
        url: str,
        headers: Optional[Mapping[str, str]] = None,
        auth: Optional[Tuple[str, str]] = None,
        timeout: Optional[float] = None,
        return_bytes: bool = False,
        allow_errors: bool = False,
        verbose: bool = True,
    ) -> Tuple[bool, Union[Dict[str, Any], bytes], int, Optional[requests.RequestException]]:
        """Perform a DELETE request with the given url and headers.

        Args:
            url: The url to DELETE.
            headers: The headers to use during the request.
            auth: A tuple containing the username and password to use in the request.
            timeout: How many seconds to wait for the server to send data before giving up.
            return_bytes: A boolean indicating if the response content should be returned
                instead of the response json.
            allow_errors: A boolean indicating if errors are allowed.
            verbose: Set this to False in order to disable printouts.

        Returns:
            A tuple containing a success boolean, the response data, status code, and any errors
        """
        return self._send_request(
            SupportedRequestTypes.DELETE,
            url,
            headers=headers,
            auth=auth,
            timeout=timeout,
            return_bytes=return_bytes,
            allow_errors=allow_errors,
            verbose=verbose,
        )

    def get(  # noqa: PLR0913
        self,
        url: str,
        headers: Optional[Mapping[str, str]] = None,
        auth: Optional[Tuple[str, str]] = None,
        timeout: Optional[float] = None,
        return_bytes: bool = False,
        allow_errors: bool = False,
        verbose: bool = True,
    ) -> Tuple[bool, Union[Dict[str, Any], bytes], int, Optional[requests.RequestException]]:
        """Perform a GET request with the given url and headers.

        Args:
            url: The url to GET.
            headers: The headers to use during the request.
            auth: A tuple containing the username and password to use in the request.
            timeout: How many seconds to wait for the server to send data before giving up.
            return_bytes: A boolean indicating if the response content should be returned
                instead of the response json.
            allow_errors: A boolean indicating if errors are allowed.
            verbose: Set this to False in order to disable printouts.

        Returns:
            A tuple containing a success boolean, the response data, status code, and any errors
        """
        return self._send_request(
            SupportedRequestTypes.GET,
            url,
            headers=headers,
            auth=auth,
            timeout=timeout,
            return_bytes=return_bytes,
            allow_errors=allow_errors,
            verbose=verbose,
        )

    def patch(  # noqa: PLR0913
        self,
        url: str,
        json_body: Dict[str, Any],
        headers: Optional[Mapping[str, str]] = None,
        auth: Optional[Tuple[str, str]] = None,
        timeout: Optional[float] = None,
        return_bytes: bool = False,
        allow_errors: bool = False,
        verbose: bool = True,
    ) -> Tuple[bool, Union[Dict[str, Any], bytes], int, Optional[requests.RequestException]]:
        """Perform a PATCH request with the given url and headers.

        Args:
            url: The url to PATCH.
            json_body: Any data to send to the url.
            headers: The headers to use during the request.
            auth: A tuple containing the username and password to use in the request.
            timeout: How many seconds to wait for the server to send data before giving up.
            return_bytes: A boolean indicating if the response content should be returned
                instead of the response json.
            allow_errors: A boolean indicating if errors are allowed.
            verbose: Set this to False in order to disable printouts.

        Returns:
            A tuple containing a success boolean, the response data, status code, and any errors
        """
        return self._send_request(
            SupportedRequestTypes.PATCH,
            url,
            json_body=json_body,
            headers=headers,
            auth=auth,
            timeout=timeout,
            return_bytes=return_bytes,
            allow_errors=allow_errors,
            verbose=verbose,
        )

    def post(  # noqa: PLR0913
        self,
        url: str,
        json_body: Optional[Dict[str, Any]] = None,
        headers: Optional[Mapping[str, str]] = None,
        auth: Optional[Tuple[str, str]] = None,
        timeout: Optional[float] = None,
        return_bytes: bool = False,
        allow_errors: bool = False,
        verbose: bool = True,
    ) -> Tuple[bool, Union[Dict[str, Any], bytes], int, Optional[requests.RequestException]]:
        """Perform a POST request with the given url and headers.

        Args:
            url: The url to POST.
            json_body: Any data to send to the url.
            headers: The headers to use during the request.
            auth: A tuple containing the username and password to use in the request.
            timeout: How many seconds to wait for the server to send data before giving up.
            return_bytes: A boolean indicating if the response content should be returned
                instead of the response json.
            allow_errors: A boolean indicating if errors are allowed.
            verbose: Set this to False in order to disable printouts.

        Returns:
            A tuple containing a success boolean, the response data, status code, and any errors
        """
        return self._send_request(
            SupportedRequestTypes.POST,
            url,
            json_body=json_body,
            headers=headers,
            auth=auth,
            timeout=timeout,
            return_bytes=return_bytes,
            allow_errors=allow_errors,
            verbose=verbose,
        )

    def put(  # noqa: PLR0913
        self,
        url: str,
        json_body: Dict[str, Any],
        headers: Optional[Mapping[str, str]] = None,
        auth: Optional[Tuple[str, str]] = None,
        timeout: Optional[float] = None,
        return_bytes: bool = False,
        allow_errors: bool = False,
        verbose: bool = True,
    ) -> Tuple[bool, Union[Dict[str, Any], bytes], int, Optional[requests.RequestException]]:
        """Perform a PUT request with the given url and headers.

        Args:
            url: The url to PUT.
            json_body: Any data to send to the url.
            headers: The headers to use during the request.
            auth: A tuple containing the username and password to use in the request.
            timeout: How many seconds to wait for the server to send data before giving up.
            return_bytes: A boolean indicating if the response content should be returned
                instead of the response json.
            allow_errors: A boolean indicating if errors are allowed.
            verbose: Set this to False in order to disable printouts.

        Returns:
            A tuple containing a success boolean, the response data, status code, and any errors
        """
        return self._send_request(
            SupportedRequestTypes.PUT,
            url,
            json_body=json_body,
            headers=headers,
            auth=auth,
            timeout=timeout,
            return_bytes=return_bytes,
            allow_errors=allow_errors,
            verbose=verbose,
        )

    def set_api_version(self, api_version: int, verbose: bool = True) -> None:
        """Set the API version used by the device.

        Args:
            api_version: The API version to use for this device
            verbose: Set this to False in order to disable printouts.
        """
        self._api_url = self._base_url + self.API_VERSIONS[api_version]
        if self._verbose and verbose:
            print(f"{self._name_and_alias} API version set to: {api_version} ({self._api_url})")

    def wait_for_api_connection(
        self,
        wait_time: float,
        sleep_seconds: int = 5,
        accept_immediate_connection: bool = False,
        verbose: bool = True,
    ) -> bool:
        """Wait for an API call to go through to the device.

        Using wait_for_network_connection does not ensure that the API network is also up.

        Args:
            wait_time: The number of seconds to wait for the API connection.
            sleep_seconds: The number of seconds to sleep in between connection attempts.
            accept_immediate_connection: A boolean indicating if a connection on the
                                         first attempt is a valid connection.
            verbose: Set this to False in order to disable printouts.

        Returns:
            A boolean indicating if an API connection was made within the given time limit.

        Raises:
            AssertionError: Indicating that the device erroneously connected on the first try.
        """
        attempt_num = 0
        api_connection = False
        if verbose:
            print_with_timestamp(f"Attempting to establish an API connection with {self._api_url}")
        start_time = time.perf_counter()
        while (time.perf_counter() - start_time) <= wait_time:
            if api_connection := self._check_api_connection():
                # pylint: disable=compare-to-zero
                if attempt_num != 0 or accept_immediate_connection:
                    break
                msg = (
                    f"An API connection was established with "
                    f"{self._api_url} on the first attempt "
                    f"when it should not have been."
                )
                raise AssertionError(msg)
            time.sleep(sleep_seconds)
            attempt_num += 1

        end_time = time.perf_counter()
        total_time = end_time - start_time

        if verbose:
            if api_connection:
                print_with_timestamp(
                    f"Successfully established an API connection with {self._api_url} "
                    f"after {total_time:.2f} seconds"
                )
            else:
                print_with_timestamp(
                    f"Unable to establish an API connection with {self._api_url} "
                    f"after {total_time:.2f} seconds"
                )
        return api_connection

    ################################################################################################
    # Private Methods
    ################################################################################################
    @abstractmethod
    def _check_api_connection(self) -> bool:
        """Make any call to the API to check if there is a connection.

        Returns:
            A boolean indicating if the call was successful.
        """
        raise NotImplementedError

    # pylint: disable=too-many-branches
    def _send_request(  # noqa: PLR0913,PLR0912,C901
        self,
        request_type: SupportedRequestTypes,
        url: str,
        json_body: Optional[Dict[str, Any]] = None,
        headers: Optional[Mapping[str, str]] = None,
        auth: Optional[Tuple[str, str]] = None,
        timeout: Optional[float] = None,
        return_bytes: bool = False,
        allow_errors: bool = False,
        verbose: bool = True,
    ) -> Tuple[bool, Union[Dict[str, Any], bytes], int, Optional[requests.RequestException]]:
        """Perform a request with the given url and headers.

        Args:
            request_type: The type of request to send.
            url: The url to use.
            json_body: Any data to send to the url.
            headers: The headers to use during the request.
            auth: A tuple containing the username and password to use in the request.
            timeout: How many seconds to wait for the server to send data before giving up.
            return_bytes: A boolean indicating if the response content should be returned
                instead of the response json.
            allow_errors: A boolean indicating if errors are allowed.
            verbose: Set this to False in order to disable printouts.

        Returns:
            A tuple containing a success boolean, the response data, status code, and any errors

        Raises:
            ValueError: Indicates that an unsupported request type was provided.
        """
        # Sanitize the url by prepending the api url if necessary
        if not url.startswith(self._base_url):
            if not url.startswith("/"):
                url = "/" + url
            if any(url.startswith(api) for api in self.API_VERSIONS.values()):
                url = self._base_url + url
            else:
                url = self._api_url + url

        response = cast(requests.Response, None)
        retval: Union[Dict[str, Any], bytes] = {}
        if self._verbose and verbose:
            print_with_timestamp(f"({self._name_and_alias}) {request_type.value} >> {url}", end="")
            if headers:
                print(f", {headers=}", end="")
            if json_body:
                print(f", {json_body=}", end="")
            print("")

        try:
            if request_type == SupportedRequestTypes.DELETE:
                response = requests.delete(url, headers=headers, auth=auth, timeout=timeout)
            elif request_type == SupportedRequestTypes.GET:
                response = requests.get(url, headers=headers, auth=auth, timeout=timeout)
            elif request_type == SupportedRequestTypes.PATCH:
                response = requests.patch(
                    url, headers=headers, auth=auth, json=json_body, timeout=timeout
                )
            elif request_type == SupportedRequestTypes.POST:
                response = requests.post(
                    url, headers=headers, auth=auth, json=json_body, timeout=timeout
                )
            elif request_type == SupportedRequestTypes.PUT:
                response = requests.put(
                    url, headers=headers, auth=auth, json=json_body, timeout=timeout
                )
            else:
                msg = f"{request_type} is an unsupported request type."
                raise ValueError(msg)
            if self._verbose and verbose:
                print_with_timestamp(f"Response from {request_type.value} >>")
                if not return_bytes:
                    print(f"{json.dumps(response.json(), indent=2)}")
                else:
                    # Print .zip files as byte strings, response.content.decode() throws errors
                    print(f"\n{response.text}")
            retval = response.content if return_bytes else response.json()
            # If the response was successful, no Exception will be raised
            response.raise_for_status()
        except requests.exceptions.RequestException as error:
            if not allow_errors:
                self.raise_failure(
                    f"A RequestException occurred (status code {response.status_code}): {error}"
                )
            try:
                status_code = response.status_code
            except AttributeError:
                try:
                    status_code = (
                        error.response.status_code  # pyright: ignore[reportOptionalMemberAccess]
                    )
                except AttributeError:
                    status_code = 503
            return False, retval, status_code, error

        return True, retval, response.status_code, None



# pylint: disable=too-many-lines
# ruff: noqa: D402,PLR0913
# pyright: reportConstantRedefinition=none
"""The buffervar commands module.

These commands are used in the following models:
SMU2601B, SMU2602B, SMU2604B, SMU2606B, SMU2611B, SMU2612B, SMU2614B, SMU2634B, SMU2635B, SMU2636B,
SMU2651A, SMU2657A

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Attributes and Functions:

::

    - bufferVar.appendmode
    - bufferVar.basetimestamp
    - bufferVar.cachemode
    - bufferVar.capacity
    - bufferVar.clear()
    - bufferVar.clearcache()
    - bufferVar.collectsourcevalues
    - bufferVar.collecttimestamps
    - bufferVar.fillcount
    - bufferVar.fillmode
    - bufferVar.measurefunctions[N]
    - bufferVar.measureranges[N]
    - bufferVar.n
    - bufferVar.readings[N]
    - bufferVar.sourcefunctions[N]
    - bufferVar.sourceoutputstates[N]
    - bufferVar.sourceranges[N]
    - bufferVar.sourcevalues[N]
    - bufferVar.statuses[N]
    - bufferVar.timestampresolution
    - bufferVar.timestamps[N]
"""
from typing import Dict, Optional, TYPE_CHECKING, Union

from .._helpers import BaseTSPCmd, DefaultDictDeviceCommunication, NoDeviceProvidedError

if TYPE_CHECKING:
    from tm_devices.drivers.pi.tsp_device import TSPDevice


#  pylint: disable=too-many-instance-attributes,too-many-public-methods
class Buffervar(BaseTSPCmd):
    """The ``bufferVar`` command tree.

    **Info:**
        - ``bufferVar``, the reading buffer; can be a dynamically allocated user-defined buffer or a
          dedicated reading buffer.

    Properties/methods:
        - ``.appendmode``: The ``bufferVar.appendmode`` attribute.
        - ``.basetimestamp``: The ``bufferVar.basetimestamp`` attribute.
        - ``.cachemode``: The ``bufferVar.cachemode`` attribute.
        - ``.capacity``: The ``bufferVar.capacity`` attribute.
        - ``.clear()``: The ``bufferVar.clear()`` function.
        - ``.clearcache()``: The ``bufferVar.clearcache()`` function.
        - ``.collectsourcevalues``: The ``bufferVar.collectsourcevalues`` attribute.
        - ``.collecttimestamps``: The ``bufferVar.collecttimestamps`` attribute.
        - ``.fillcount``: The ``bufferVar.fillcount`` attribute.
        - ``.fillmode``: The ``bufferVar.fillmode`` attribute.
        - ``.measurefunctions``: The ``bufferVar.measurefunctions[N]`` attribute.
        - ``.measureranges``: The ``bufferVar.measureranges[N]`` attribute.
        - ``.n``: The ``bufferVar.n`` attribute.
        - ``.readings``: The ``bufferVar.readings[N]`` attribute.
        - ``.sourcefunctions``: The ``bufferVar.sourcefunctions[N]`` attribute.
        - ``.sourceoutputstates``: The ``bufferVar.sourceoutputstates[N]`` attribute.
        - ``.sourceranges``: The ``bufferVar.sourceranges[N]`` attribute.
        - ``.sourcevalues``: The ``bufferVar.sourcevalues[N]`` attribute.
        - ``.statuses``: The ``bufferVar.statuses[N]`` attribute.
        - ``.timestampresolution``: The ``bufferVar.timestampresolution`` attribute.
        - ``.timestamps``: The ``bufferVar.timestamps[N]`` attribute.
    """

    def __init__(self, device: Optional["TSPDevice"] = None, cmd_syntax: str = "bufferVar") -> None:
        super().__init__(device, cmd_syntax)
        self._measurefunctions: Dict[int, Union[str, float]] = DefaultDictDeviceCommunication(
            cmd_syntax=f"{self._cmd_syntax}.measurefunctions[{{key}}]",
            query_syntax=f"print({self._cmd_syntax}.measurefunctions[{{key}}])",
            device=self._device,
        )
        self._measureranges: Dict[int, Union[str, float]] = DefaultDictDeviceCommunication(
            cmd_syntax=f"{self._cmd_syntax}.measureranges[{{key}}]",
            query_syntax=f"print({self._cmd_syntax}.measureranges[{{key}}])",
            device=self._device,
        )
        self._readings: Dict[int, Union[str, float]] = DefaultDictDeviceCommunication(
            cmd_syntax=f"{self._cmd_syntax}.readings[{{key}}]",
            query_syntax=f"print({self._cmd_syntax}.readings[{{key}}])",
            device=self._device,
        )
        self._sourcefunctions: Dict[int, Union[str, float]] = DefaultDictDeviceCommunication(
            cmd_syntax=f"{self._cmd_syntax}.sourcefunctions[{{key}}]",
            query_syntax=f"print({self._cmd_syntax}.sourcefunctions[{{key}}])",
            device=self._device,
        )
        self._sourceoutputstates: Dict[int, Union[str, float]] = DefaultDictDeviceCommunication(
            cmd_syntax=f"{self._cmd_syntax}.sourceoutputstates[{{key}}]",
            query_syntax=f"print({self._cmd_syntax}.sourceoutputstates[{{key}}])",
            device=self._device,
        )
        self._sourceranges: Dict[int, Union[str, float]] = DefaultDictDeviceCommunication(
            cmd_syntax=f"{self._cmd_syntax}.sourceranges[{{key}}]",
            query_syntax=f"print({self._cmd_syntax}.sourceranges[{{key}}])",
            device=self._device,
        )
        self._sourcevalues: Dict[int, Union[str, float]] = DefaultDictDeviceCommunication(
            cmd_syntax=f"{self._cmd_syntax}.sourcevalues[{{key}}]",
            query_syntax=f"print({self._cmd_syntax}.sourcevalues[{{key}}])",
            device=self._device,
        )
        self._statuses: Dict[int, Union[str, float]] = DefaultDictDeviceCommunication(
            cmd_syntax=f"{self._cmd_syntax}.statuses[{{key}}]",
            query_syntax=f"print({self._cmd_syntax}.statuses[{{key}}])",
            device=self._device,
        )
        self._timestamps: Dict[int, Union[str, float]] = DefaultDictDeviceCommunication(
            cmd_syntax=f"{self._cmd_syntax}.timestamps[{{key}}]",
            query_syntax=f"print({self._cmd_syntax}.timestamps[{{key}}])",
            device=self._device,
        )

    @property
    def appendmode(self) -> str:
        """Access the ``bufferVar.appendmode`` attribute.

        **Description:**
            - This attribute sets the state of the append mode of the reading buffer.

        **Usage:**
            - Accessing this property will send the ``print(bufferVar.appendmode)`` query.
            - Setting this property to a value will send the ``bufferVar.appendmode = value``
              command.

        **TSP Syntax:**

        ::

            - bufferVar.appendmode = value
            - print(bufferVar.appendmode)

        **Info:**
            - ``bufferVar``, the reading buffer; can be a dynamically allocated user-defined buffer
              or a dedicated reading buffer.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".appendmode"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.appendmode)"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.appendmode`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @appendmode.setter
    def appendmode(self, value: Union[str, float]) -> None:
        """Access the ``bufferVar.appendmode`` attribute.

        **Description:**
            - This attribute sets the state of the append mode of the reading buffer.

        **Usage:**
            - Accessing this property will send the ``print(bufferVar.appendmode)`` query.
            - Setting this property to a value will send the ``bufferVar.appendmode = value``
              command.

        **TSP Syntax:**

        ::

            - bufferVar.appendmode = value
            - print(bufferVar.appendmode)

        **Info:**
            - ``bufferVar``, the reading buffer; can be a dynamically allocated user-defined buffer
              or a dedicated reading buffer.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_verification_enabled:  # type: ignore[union-attr]
                self._device.set_and_check(  # type: ignore[union-attr]
                    self._cmd_syntax + ".appendmode", value
                )
            else:
                self._device.write(  # type: ignore[union-attr]
                    f"{self._cmd_syntax}.appendmode = {value}"
                )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.appendmode`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @property
    def basetimestamp(self) -> str:
        """Access the ``bufferVar.basetimestamp`` attribute.

        **Description:**
            - This attribute contains the timestamp that indicates when the first reading was stored
              in the buffer.

        **Usage:**
            - Accessing this property will send the ``print(bufferVar.basetimestamp)`` query.

        **TSP Syntax:**

        ::

            - print(bufferVar.basetimestamp)

        **Info:**
            - ``bufferVar``, the reading buffer; can be a dynamically allocated buffer
              (user-defined), or a dedicated reading buffer (such as smua.nvbuffer1).

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".basetimestamp"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.basetimestamp)"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.basetimestamp`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @property
    def cachemode(self) -> str:
        """Access the ``bufferVar.cachemode`` attribute.

        **Description:**
            - This attribute enables or disables the reading buffer cache (on or off).

        **Usage:**
            - Accessing this property will send the ``print(bufferVar.cachemode)`` query.
            - Setting this property to a value will send the ``bufferVar.cachemode = value``
              command.

        **TSP Syntax:**

        ::

            - bufferVar.cachemode = value
            - print(bufferVar.cachemode)

        **Info:**
            - ``bufferVar``, the reading buffer; can be a dynamically allocated user-defined buffer
              or a dedicated reading buffer.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".cachemode"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.cachemode)"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.cachemode`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @cachemode.setter
    def cachemode(self, value: Union[str, float]) -> None:
        """Access the ``bufferVar.cachemode`` attribute.

        **Description:**
            - This attribute enables or disables the reading buffer cache (on or off).

        **Usage:**
            - Accessing this property will send the ``print(bufferVar.cachemode)`` query.
            - Setting this property to a value will send the ``bufferVar.cachemode = value``
              command.

        **TSP Syntax:**

        ::

            - bufferVar.cachemode = value
            - print(bufferVar.cachemode)

        **Info:**
            - ``bufferVar``, the reading buffer; can be a dynamically allocated user-defined buffer
              or a dedicated reading buffer.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_verification_enabled:  # type: ignore[union-attr]
                self._device.set_and_check(  # type: ignore[union-attr]
                    self._cmd_syntax + ".cachemode", value
                )
            else:
                self._device.write(  # type: ignore[union-attr]
                    f"{self._cmd_syntax}.cachemode = {value}"
                )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.cachemode`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @property
    def capacity(self) -> str:
        """Access the ``bufferVar.capacity`` attribute.

        **Description:**
            - This attribute sets the number of readings a buffer can store.

        **Usage:**
            - Accessing this property will send the ``print(bufferVar.capacity)`` query.

        **TSP Syntax:**

        ::

            - print(bufferVar.capacity)

        **Info:**
            - ``bufferVar``, the reading buffer; can be a dynamically allocated user-defined buffer
              or a dedicated reading buffer.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".capacity"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.capacity)"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.capacity`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @property
    def collectsourcevalues(self) -> str:
        """Access the ``bufferVar.collectsourcevalues`` attribute.

        **Description:**
            - This attribute sets whether or not source values are stored with the readings in the
              buffer.

        **Usage:**
            - Accessing this property will send the ``print(bufferVar.collectsourcevalues)`` query.
            - Setting this property to a value will send the
              ``bufferVar.collectsourcevalues = value`` command.

        **TSP Syntax:**

        ::

            - bufferVar.collectsourcevalues = value
            - print(bufferVar.collectsourcevalues)

        **Info:**
            - ``bufferVar``, the reading buffer; can be a dynamically allocated buffer
              (user-defined), or a dedicated reading buffer (such as smua.nvbuffer1).

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".collectsourcevalues"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.collectsourcevalues)"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.collectsourcevalues`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @collectsourcevalues.setter
    def collectsourcevalues(self, value: Union[str, float]) -> None:
        """Access the ``bufferVar.collectsourcevalues`` attribute.

        **Description:**
            - This attribute sets whether or not source values are stored with the readings in the
              buffer.

        **Usage:**
            - Accessing this property will send the ``print(bufferVar.collectsourcevalues)`` query.
            - Setting this property to a value will send the
              ``bufferVar.collectsourcevalues = value`` command.

        **TSP Syntax:**

        ::

            - bufferVar.collectsourcevalues = value
            - print(bufferVar.collectsourcevalues)

        **Info:**
            - ``bufferVar``, the reading buffer; can be a dynamically allocated buffer
              (user-defined), or a dedicated reading buffer (such as smua.nvbuffer1).

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_verification_enabled:  # type: ignore[union-attr]
                self._device.set_and_check(  # type: ignore[union-attr]
                    self._cmd_syntax + ".collectsourcevalues", value
                )
            else:
                self._device.write(  # type: ignore[union-attr]
                    f"{self._cmd_syntax}.collectsourcevalues = {value}"
                )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.collectsourcevalues`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @property
    def collecttimestamps(self) -> str:
        """Access the ``bufferVar.collecttimestamps`` attribute.

        **Description:**
            - This attribute sets whether or not timestamp values are stored with the readings in
              the buffer.

        **Usage:**
            - Accessing this property will send the ``print(bufferVar.collecttimestamps)`` query.
            - Setting this property to a value will send the ``bufferVar.collecttimestamps = value``
              command.

        **TSP Syntax:**

        ::

            - bufferVar.collecttimestamps = value
            - print(bufferVar.collecttimestamps)

        **Info:**
            - ``bufferVar``, the reading buffer; can be a dynamically allocated user-defined buffer
              or a dedicated reading buffer.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".collecttimestamps"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.collecttimestamps)"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.collecttimestamps`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @collecttimestamps.setter
    def collecttimestamps(self, value: Union[str, float]) -> None:
        """Access the ``bufferVar.collecttimestamps`` attribute.

        **Description:**
            - This attribute sets whether or not timestamp values are stored with the readings in
              the buffer.

        **Usage:**
            - Accessing this property will send the ``print(bufferVar.collecttimestamps)`` query.
            - Setting this property to a value will send the ``bufferVar.collecttimestamps = value``
              command.

        **TSP Syntax:**

        ::

            - bufferVar.collecttimestamps = value
            - print(bufferVar.collecttimestamps)

        **Info:**
            - ``bufferVar``, the reading buffer; can be a dynamically allocated user-defined buffer
              or a dedicated reading buffer.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_verification_enabled:  # type: ignore[union-attr]
                self._device.set_and_check(  # type: ignore[union-attr]
                    self._cmd_syntax + ".collecttimestamps", value
                )
            else:
                self._device.write(  # type: ignore[union-attr]
                    f"{self._cmd_syntax}.collecttimestamps = {value}"
                )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.collecttimestamps`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @property
    def fillcount(self) -> str:
        """Access the ``bufferVar.fillcount`` attribute.

        **Description:**
            - This attribute sets the reading buffer fill count.

        **Usage:**
            - Accessing this property will send the ``print(bufferVar.fillcount)`` query.
            - Setting this property to a value will send the ``bufferVar.fillcount = value``
              command.

        **TSP Syntax:**

        ::

            - bufferVar.fillcount = value
            - print(bufferVar.fillcount)

        **Info:**
            - ``bufferVar``, the reading buffer; can be a dynamically allocated buffer
              (user-defined), or a dedicated reading buffer (such as smua.nvbuffer1).

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".fillcount"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.fillcount)"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.fillcount`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @fillcount.setter
    def fillcount(self, value: Union[str, float]) -> None:
        """Access the ``bufferVar.fillcount`` attribute.

        **Description:**
            - This attribute sets the reading buffer fill count.

        **Usage:**
            - Accessing this property will send the ``print(bufferVar.fillcount)`` query.
            - Setting this property to a value will send the ``bufferVar.fillcount = value``
              command.

        **TSP Syntax:**

        ::

            - bufferVar.fillcount = value
            - print(bufferVar.fillcount)

        **Info:**
            - ``bufferVar``, the reading buffer; can be a dynamically allocated buffer
              (user-defined), or a dedicated reading buffer (such as smua.nvbuffer1).

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_verification_enabled:  # type: ignore[union-attr]
                self._device.set_and_check(  # type: ignore[union-attr]
                    self._cmd_syntax + ".fillcount", value
                )
            else:
                self._device.write(  # type: ignore[union-attr]
                    f"{self._cmd_syntax}.fillcount = {value}"
                )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.fillcount`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @property
    def fillmode(self) -> str:
        """Access the ``bufferVar.fillmode`` attribute.

        **Description:**
            - This attribute sets the reading buffer fill mode.

        **Usage:**
            - Accessing this property will send the ``print(bufferVar.fillmode)`` query.
            - Setting this property to a value will send the ``bufferVar.fillmode = value`` command.

        **TSP Syntax:**

        ::

            - bufferVar.fillmode = value
            - print(bufferVar.fillmode)

        **Info:**
            - ``bufferVar``, the reading buffer; can be a dynamically allocated buffer
              (user-defined), or a dedicated reading buffer (such as smua.nvbuffer1).

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".fillmode"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.fillmode)"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.fillmode`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @fillmode.setter
    def fillmode(self, value: Union[str, float]) -> None:
        """Access the ``bufferVar.fillmode`` attribute.

        **Description:**
            - This attribute sets the reading buffer fill mode.

        **Usage:**
            - Accessing this property will send the ``print(bufferVar.fillmode)`` query.
            - Setting this property to a value will send the ``bufferVar.fillmode = value`` command.

        **TSP Syntax:**

        ::

            - bufferVar.fillmode = value
            - print(bufferVar.fillmode)

        **Info:**
            - ``bufferVar``, the reading buffer; can be a dynamically allocated buffer
              (user-defined), or a dedicated reading buffer (such as smua.nvbuffer1).

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_verification_enabled:  # type: ignore[union-attr]
                self._device.set_and_check(  # type: ignore[union-attr]
                    self._cmd_syntax + ".fillmode", value
                )
            else:
                self._device.write(  # type: ignore[union-attr]
                    f"{self._cmd_syntax}.fillmode = {value}"
                )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.fillmode`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @property
    def measurefunctions(self) -> Dict[int, Union[str, float]]:
        """Access the ``bufferVar.measurefunctions[N]`` attribute.

        **Description:**
            - This attribute contains the measurement function that was used to acquire a reading
              stored in a specified reading buffer.

        **Usage:**
            - Accessing an item from this property will send the
              ``print(bufferVar.measurefunctions[N])`` query.

        **TSP Syntax:**

        ::

            - print(bufferVar.measurefunctions[N])

        **Info:**
            - ``bufferVar``, the reading buffer; can be a dynamically allocated buffer
              (user-defined), or a dedicated reading buffer (such as smua.nvbuffer1).
            - ``N``, the reading number (1 to bufferVar.n).

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        return self._measurefunctions

    @property
    def measureranges(self) -> Dict[int, Union[str, float]]:
        """Access the ``bufferVar.measureranges[N]`` attribute.

        **Description:**
            - This attribute contains the measurement range values that were used for readings
              stored in a specified buffer.

        **Usage:**
            - Accessing an item from this property will send the
              ``print(bufferVar.measureranges[N])`` query.

        **TSP Syntax:**

        ::

            - print(bufferVar.measureranges[N])

        **Info:**
            - ``bufferVar``, the reading buffer; can be a dynamically allocated buffer
              (user-defined), or a dedicated reading buffer (such as smua.nvbuffer1).
            - ``N``, the reading number (1 to bufferVar.n).

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        return self._measureranges

    @property
    def n(self) -> str:
        """Access the ``bufferVar.n`` attribute.

        **Description:**
            - This attribute contains the number of readings in the buffer.

        **Usage:**
            - Accessing this property will send the ``print(bufferVar.n)`` query.

        **TSP Syntax:**

        ::

            - print(bufferVar.n)

        **Info:**
            - ``bufferVar``, the reading buffer; can be a dynamically allocated user-defined buffer
              or a dedicated reading buffer.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".n"
            return self._device.query(f"print({self._cmd_syntax}.n)")  # type: ignore[union-attr]
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.n`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @property
    def readings(self) -> Dict[int, Union[str, float]]:
        """Access the ``bufferVar.readings[N]`` attribute.

        **Description:**
            - This attribute contains the readings stored in a specified reading buffer.

        **Usage:**
            - Accessing an item from this property will send the ``print(bufferVar.readings[N])``
              query.

        **TSP Syntax:**

        ::

            - print(bufferVar.readings[N])

        **Info:**
            - ``bufferVar``, the reading buffer; can be a dynamically allocated user-defined buffer
              or a dedicated reading buffer.
            - ``N``, the reading number N; can be any value from 1 to the number of readings in the
              buffer; use the bufferVar.n command to determine the number of readings in the buffer.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        return self._readings

    @property
    def sourcefunctions(self) -> Dict[int, Union[str, float]]:
        """Access the ``bufferVar.sourcefunctions[N]`` attribute.

        **Description:**
            - This attribute contains the source function that was being used when the readings were
              stored in a specified reading buffer.

        **Usage:**
            - Accessing an item from this property will send the
              ``print(bufferVar.sourcefunctions[N])`` query.

        **TSP Syntax:**

        ::

            - print(bufferVar.sourcefunctions[N])

        **Info:**
            - ``bufferVar``, the reading buffer; can be a dynamically allocated buffer
              (user-defined), or a dedicated reading buffer (such as smua.nvbuffer1).
            - ``N``, the reading number (1 to bufferVar.n).

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        return self._sourcefunctions

    @property
    def sourceoutputstates(self) -> Dict[int, Union[str, float]]:
        """Access the ``bufferVar.sourceoutputstates[N]`` attribute.

        **Description:**
            - This attribute indicates the state of the source output for readings that are stored
              in a specified buffer.

        **Usage:**
            - Accessing an item from this property will send the
              ``print(bufferVar.sourceoutputstates[N])`` query.

        **TSP Syntax:**

        ::

            - print(bufferVar.sourceoutputstates[N])

        **Info:**
            - ``bufferVar``, the reading buffer; can be a dynamically allocated buffer
              (user-defined), or a dedicated reading buffer (such as smua.nvbuffer1).
            - ``N``, the reading number (1 to bufferVar.n).

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        return self._sourceoutputstates

    @property
    def sourceranges(self) -> Dict[int, Union[str, float]]:
        """Access the ``bufferVar.sourceranges[N]`` attribute.

        **Description:**
            - This attribute contains the source range that was used for readings stored in a
              specified reading buffer.

        **Usage:**
            - Accessing an item from this property will send the
              ``print(bufferVar.sourceranges[N])`` query.

        **TSP Syntax:**

        ::

            - print(bufferVar.sourceranges[N])

        **Info:**
            - ``bufferVar``, the reading buffer; can be a dynamically allocated buffer
              (user-defined), or a dedicated reading buffer (such as smua.nvbuffer1).
            - ``N``, the reading number (1 to bufferVar.n).

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        return self._sourceranges

    @property
    def sourcevalues(self) -> Dict[int, Union[str, float]]:
        """Access the ``bufferVar.sourcevalues[N]`` attribute.

        **Description:**
            - When enabled by the bufferVar.collectsourcevalues attribute, this attribute contains
              the source levels being output when readings in the reading buffer were acquired.

        **Usage:**
            - Accessing an item from this property will send the
              ``print(bufferVar.sourcevalues[N])`` query.

        **TSP Syntax:**

        ::

            - print(bufferVar.sourcevalues[N])

        **Info:**
            - ``bufferVar``, the reading buffer; can be a dynamically allocated buffer
              (user-defined) or a dedicated reading buffer (such as smua.nvbuffer1).
            - ``N``, the reading number (1 to bufferVar.n).

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        return self._sourcevalues

    @property
    def statuses(self) -> Dict[int, Union[str, float]]:
        """Access the ``bufferVar.statuses[N]`` attribute.

        **Description:**
            - This attribute contains the status values of readings in the reading buffer.

        **Usage:**
            - Accessing an item from this property will send the ``print(bufferVar.statuses[N])``
              query.

        **TSP Syntax:**

        ::

            - print(bufferVar.statuses[N])

        **Info:**
            - ``bufferVar``, the reading buffer; can be a dynamically allocated user-defined buffer
              or a dedicated reading buffer.
            - ``N``, the reading number N; can be any value from 1 to the number of readings in the
              buffer; use the bufferVar.n command to determine the number of readings in the buffer.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        return self._statuses

    @property
    def timestampresolution(self) -> str:
        """Access the ``bufferVar.timestampresolution`` attribute.

        **Description:**
            - This attribute contains the resolution of the timestamp.

        **Usage:**
            - Accessing this property will send the ``print(bufferVar.timestampresolution)`` query.
            - Setting this property to a value will send the
              ``bufferVar.timestampresolution = value`` command.

        **TSP Syntax:**

        ::

            - bufferVar.timestampresolution = value
            - print(bufferVar.timestampresolution)

        **Info:**
            - ``resolution``, the timestamp resolution in seconds (minimum 1 µs; rounded to an even
              power of 2 µs).
            - ``bufferVar``, the reading buffer; can be a dynamically allocated user-defined buffer
              or a dedicated reading buffer.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".timestampresolution"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.timestampresolution)"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.timestampresolution`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @timestampresolution.setter
    def timestampresolution(self, value: Union[str, float]) -> None:
        """Access the ``bufferVar.timestampresolution`` attribute.

        **Description:**
            - This attribute contains the resolution of the timestamp.

        **Usage:**
            - Accessing this property will send the ``print(bufferVar.timestampresolution)`` query.
            - Setting this property to a value will send the
              ``bufferVar.timestampresolution = value`` command.

        **TSP Syntax:**

        ::

            - bufferVar.timestampresolution = value
            - print(bufferVar.timestampresolution)

        **Info:**
            - ``resolution``, the timestamp resolution in seconds (minimum 1 µs; rounded to an even
              power of 2 µs).
            - ``bufferVar``, the reading buffer; can be a dynamically allocated user-defined buffer
              or a dedicated reading buffer.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_verification_enabled:  # type: ignore[union-attr]
                self._device.set_and_check(  # type: ignore[union-attr]
                    self._cmd_syntax + ".timestampresolution", value
                )
            else:
                self._device.write(  # type: ignore[union-attr]
                    f"{self._cmd_syntax}.timestampresolution = {value}"
                )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.timestampresolution`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @property
    def timestamps(self) -> Dict[int, Union[str, float]]:
        """Access the ``bufferVar.timestamps[N]`` attribute.

        **Description:**
            - When enabled by the bufferVar.collecttimestamps attribute, this attribute contains the
              timestamp when each reading saved in the specified reading buffer occurred.

        **Usage:**
            - Accessing an item from this property will send the ``print(bufferVar.timestamps[N])``
              query.

        **TSP Syntax:**

        ::

            - print(bufferVar.timestamps[N])

        **Info:**
            - ``bufferVar``, the reading buffer; can be a dynamically allocated user-defined buffer
              or a dedicated reading buffer.
            - ``N``, the reading number (1 to bufferVar.n).

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        return self._timestamps

    def clear(self) -> None:
        """Run the ``bufferVar.clear()`` function.

        **Description:**
            - This function empties the buffer.

        **TSP Syntax:**

        ::

            - bufferVar.clear()

        **Info:**
            - ``bufferVar``, the reading buffer; can be a dynamically allocated user-defined buffer
              or a dedicated reading buffer.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            self._device.write(f"{self._cmd_syntax}.clear()")  # type: ignore[union-attr]
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.clear()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def clearcache(self) -> None:
        """Run the ``bufferVar.clearcache()`` function.

        **Description:**
            - This function clears the cache.

        **TSP Syntax:**

        ::

            - bufferVar.clearcache()

        **Info:**
            - ``bufferVar``, the reading buffer; can be a dynamically allocated user-defined buffer
              or a dedicated reading buffer.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            self._device.write(f"{self._cmd_syntax}.clearcache()")  # type: ignore[union-attr]
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.clearcache()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error



"""The system commands module.

These commands are used in the following models:
DPO5KB, DPO70KC, DPO70KD, DPO70KDX, DPO70KSX, DPO7KC, DSA70KC, DSA70KD, MSO5KB, MSO70KC, MSO70KDX

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:

::

    - SYSTem:SETup <Data Block>
    - SYSTem:SETup?
"""
from typing import Optional, TYPE_CHECKING

from .._helpers import SCPICmdRead, SCPICmdWrite

if TYPE_CHECKING:
    from tm_devices.drivers.pi.pi_device import PIDevice


class SystemSetup(SCPICmdWrite, SCPICmdRead):
    """The ``SYSTem:SETup`` command.

    **Description:**
        - This command configures the oscilloscope's data block as defined by the IEEE 488.2
          standard. The data block contains a compressed zip file. The query form of this command
          queries the block data containing the oscilloscope's current setup.

    **Usage:**
        - Using the ``.query()`` method will send the ``SYSTem:SETup?`` query.
        - Using the ``.verify(value)`` method will send the ``SYSTem:SETup?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``SYSTem:SETup value`` command.

    **SCPI Syntax:**

    ::

        - SYSTem:SETup <Data Block>
        - SYSTem:SETup?
    """


class System(SCPICmdRead):
    """The ``SYSTem`` command tree.

    **Usage:**
        - Using the ``.query()`` method will send the ``SYSTem?`` query.
        - Using the ``.verify(value)`` method will send the ``SYSTem?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.setup``: The ``SYSTem:SETup`` command.
    """

    def __init__(self, device: Optional["PIDevice"] = None, cmd_syntax: str = "SYSTem") -> None:
        super().__init__(device, cmd_syntax)
        self._setup = SystemSetup(device, f"{self._cmd_syntax}:SETup")

    @property
    def setup(self) -> SystemSetup:
        """Return the ``SYSTem:SETup`` command.

        **Description:**
            - This command configures the oscilloscope's data block as defined by the IEEE 488.2
              standard. The data block contains a compressed zip file. The query form of this
              command queries the block data containing the oscilloscope's current setup.

        **Usage:**
            - Using the ``.query()`` method will send the ``SYSTem:SETup?`` query.
            - Using the ``.verify(value)`` method will send the ``SYSTem:SETup?`` query and raise an
              AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``SYSTem:SETup value`` command.

        **SCPI Syntax:**

        ::

            - SYSTem:SETup <Data Block>
            - SYSTem:SETup?
        """
        return self._setup



# ruff: noqa: D402,PLR0913
# pyright: reportConstantRedefinition=none
"""The memory commands module.

These commands are used in the following models:
SS3706A

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Attributes and Functions:

::

    - memory.available()
    - memory.used()
"""
from typing import Optional, TYPE_CHECKING

from .._helpers import BaseTSPCmd, NoDeviceProvidedError

if TYPE_CHECKING:
    from tm_devices.drivers.pi.tsp_device import TSPDevice


class Memory(BaseTSPCmd):
    """The ``memory`` command tree.

    Properties/methods:
        - ``.available()``: The ``memory.available()`` function.
        - ``.used()``: The ``memory.used()`` function.
    """

    def __init__(self, device: Optional["TSPDevice"] = None, cmd_syntax: str = "memory") -> None:
        super().__init__(device, cmd_syntax)

    def available(self) -> str:
        """Run the ``memory.available()`` function.

        **Description:**
            - This function reads and returns the amount of memory that is available in the
              instrument overall for storing user scripts and channel patterns and for user-defined
              DMM configurations.

        **TSP Syntax:**

        ::

            - memory.available()

        Returns:
            The result of the function call.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.available())"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.available()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def used(self) -> str:
        """Run the ``memory.used()`` function.

        **Description:**
            - This function reports the amount of memory used in the instrument overall and for user
              scripts, storing channel patterns, and storing user DMM configurations.

        **TSP Syntax:**

        ::

            - memory.used()

        Returns:
            The result of the function call.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.used())"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.used()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error



"""DPO70KSX device driver module."""
from tm_devices.commands import DPO70KSXMixin
from tm_devices.drivers.pi.scopes.tekscope_5k_7k_70k.dpo70k import DPO70K


class DPO70KSX(DPO70KSXMixin, DPO70K):
    """DPO70KSX device driver."""
